package cz.fi.muni.pa165.entity;

import cz.fi.muni.pa165.enums.CountryEnum;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.TestInstance;

import org.junit.jupiter.api.Test;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import java.util.Date;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNull;

@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class PlayerEntityTest {

    private EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");

    private TeamEntity team;
    private PlayerEntity playerA;
    private PlayerEntity playerB;
    private PlayerEntity gambler;
    private Date BIRTH_DATE = new Date(1645568542000L);  // 22.2.2022  22:22:22

    @BeforeAll
    void init() {
        team = new TeamEntity();
        team.setName("Autisti");
        playerA = new PlayerEntity("Player A");
        playerB = new PlayerEntity("Player B");
        gambler = new PlayerEntity("SlotMachiner");

        team.addPlayerToTheTeam(playerA);
        team.addPlayerToTheTeam(playerB);
        playerA.setTeam(team);
        playerB.setTeam(team);
        gambler.setKillStat(1000);
        gambler.setAssistStat(500);
        gambler.setDeathStat(0);
        gambler.setCountry(CountryEnum.INDIA);
        gambler.setBirth(BIRTH_DATE);

        EntityManager em = emf.createEntityManager();
        try {
            em.getTransaction().begin();
            em.persist(team);
            em.persist(playerA);
            em.persist(playerB);
            em.persist(gambler);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void attributesTest() {
        PlayerEntity player;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            player = em.find(PlayerEntity.class, gambler.getId());
            assertNull(player.getTeam());
            assertEquals(player.getKillStat(), 1000);
            assertEquals(player.getAssistStat(), 500);
            assertEquals(player.getDeathStat(), 0);
            assertEquals(player.getCountry(), CountryEnum.INDIA);
            assertEquals(player.getBirth(), BIRTH_DATE);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipPlayerA() {
        PlayerEntity player;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            player = em.find(PlayerEntity.class, playerA.getId());
            assertEquals(player.getTeam(), team);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipPlayerB() {
        PlayerEntity player;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            player = em.find(PlayerEntity.class, playerB.getId());
            assertEquals(player.getTeam(), team);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipTeam() {
        TeamEntity team;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            team = em.find(TeamEntity.class, this.team.getId());
            assertEquals(team.getPlayerList().size(), 2);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }
}
